package com.ims.eval.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.MultipleBrandMapper;
import com.ims.eval.entity.*;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IBinSectionService;
import com.ims.eval.service.IMultipleBrandService;
import com.ims.eval.service.IOrganizationStructureService;
import com.ims.eval.util.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-05-22
 */
@Service
public class MultipleBrandServiceImpl extends ServiceImpl<MultipleBrandMapper, MultipleBrand> implements IMultipleBrandService {

	@Autowired
	private IOrganizationStructureService organizationStructureService;

	@Autowired
	private IBinSectionService binSectionService;

	@Override
	public void  initData(){
		initTree();
		initTree2();
	}

	private void initTree() {
		//按照单位营业收入配置
		IPage<MultipleBrand> tree = getMultipleBranTree(1, 1000, null, null, "", "", "", null);
		List<MultipleBrand> records = tree.getRecords();

		for (MultipleBrand record : records) {
			CacheContext.brandMap.put(record.getOrganizationId(), record);
			if (record.getChildren() == null) continue;
			for (MultipleBrand child : record.getChildren()) {
				CacheContext.brandMap.put(child.getOrganizationId(), child);
			}
		}
	}


	private void initTree2() {
		//按照单位营业收入配置
		IPage<MultipleBrand> tree = getMultipleBranTree(1, 1000, null, null, "", "", "", null);
		List<MultipleBrand> records = tree.getRecords();
		Map<String, List<MultipleBrand>> collect = records.stream().peek(mb -> {
			if ("NDKP".equals(mb.getCheckCycle())) {
				mb.setCheckCycle(mb.getCheckCycle() + "_" + mb.getYear() + "_" + mb.getMonth());
			} else {
				mb.setCheckCycle(mb.getCheckCycle() + "_" + mb.getYear() + "_" + mb.getMonth());
			}
		}).collect(Collectors.groupingBy(MultipleBrand::getCheckCycle));

		for (Map.Entry<String, List<MultipleBrand>> entry : collect.entrySet()) {
			Map<String, MultipleBrand> aaa = new HashMap<>();
			for (MultipleBrand record : entry.getValue()) {
				aaa.put(record.getOrganizationId(), record);
				if (record.getChildren() == null) {
					continue;
				}
				for (MultipleBrand child : record.getChildren()) {
					aaa.put(child.getOrganizationId(), child);
				}
			}
			CacheContext.groupBrandMap.put(entry.getKey(), aaa);
		}
	}



	@Override
	public IPage<MultipleBrand> getMultipleBranTree(Integer pageNum, Integer pageSize, String id, String parentId, String binSection, String checkCycle, String year, String month) {
		Page<Indicator> page = new Page<>(pageNum, pageSize);
		IPage<MultipleBrand> list = baseMapper.selectMultipleBranTree(page, id, "0", binSection, checkCycle, year, month);
		List<MultipleBrand> list2 = baseMapper.selectMultipleBranList(id, "", "", checkCycle, year, month);

		List<MultipleBrand> tree = convert(list.getRecords(), list2);
		list.setRecords(tree);
		return list;
	}

	@Override
	public List<MultipleBrand> getMultipleBranList(String id, String parentId, String binSection, String checkCycle, String year, String month) {
		List<MultipleBrand> list = baseMapper.selectMultipleBranList(id, parentId, binSection, checkCycle, year, month);
		return list;
	}

	@Override
	public List<MultipleBrand> getMultipleBranList2(String organizationId, String parentId, String binSection, String checkCycle, String year, String month) {
		List<MultipleBrand> list = baseMapper.selectMultipleBranList2(organizationId, parentId, binSection, checkCycle, year, month);
		return list;
	}

	@Override
	public void exportExcel(String binSection, String checkCycle, String year, HttpServletResponse response) throws IOException {
		// 设置响应头信息，以附件形式下载
		response.setHeader("Content-Disposition", "attachment; filename=multiple_brand.xlsx");
		response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		// 创建新的Excel工作簿
		Workbook workbook = new XSSFWorkbook();

		// 创建新的工作表
		Sheet sheet = workbook.createSheet("Sheet1");

		// 创建表头
		Row headerRow = sheet.createRow(0);
		List<String> headers = Arrays.asList("单位名称", "业务属性", "考评周期", "利润金额", "分数", "年", "月", "备注");
		for (int i = 0; i < headers.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellValue(headers.get(i));
		}

		//行数据
		List<MultipleBrand> firstOrderList = baseMapper.selectMultipleBranList("", "0", binSection, checkCycle, year, "");
		List<MultipleBrand> secondLevelList = baseMapper.selectMultipleBranList("", "", "", checkCycle, year, "");
		List<MultipleBrand> multipleBrandList = convert(firstOrderList, secondLevelList);

		// 填充数据到工作表
		int rowIndex = 1; // 从第二行开始填充数据（第一行是表头）
		for (MultipleBrand rootMultipleBrand : multipleBrandList) {
			Row parentRow = sheet.createRow(rowIndex++);
			writeRow(parentRow, rootMultipleBrand);

			for (MultipleBrand childMultipleBrand : rootMultipleBrand.getChildren()) {
				Row childRow = sheet.createRow(rowIndex++);
				writeRow(childRow, childMultipleBrand);
			}

			rowIndex++;
		}

		// 自动调整列宽
		for (int i = 0; i < headers.size(); i++) {
			sheet.autoSizeColumn(i);
		}

		// 将工作簿写入到输出流
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
			workbook.write(outputStream);
			response.getOutputStream().write(outputStream.toByteArray());
			response.getOutputStream().flush();
		}

		// 关闭工作簿
		workbook.close();
	}

	@Override
	public void importTemplate(HttpServletResponse response) throws IOException {
		// 设置响应头信息，以附件形式下载
		response.setHeader("Content-Disposition", "attachment; filename=multiple_brand.xlsx");
		response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		// 创建新的Excel工作簿
		Workbook workbook = new XSSFWorkbook();

		// 创建新的工作表
		Sheet sheet = workbook.createSheet("Sheet1");

		// 创建表头
		Row headerRow = sheet.createRow(0);
		List<String> headers = Arrays.asList("单位名称", "上级单位名称", "业务属性", "考评周期", "利润金额", "分数", "年", "月", "备注");
		for (int i = 0; i < headers.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellValue(headers.get(i));
		}

		// 将工作簿写入到输出流
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
			workbook.write(outputStream);
			response.getOutputStream().write(outputStream.toByteArray());
			response.getOutputStream().flush();
		}

		// 关闭工作簿
		workbook.close();
	}

	@Override
	public R importExcel(MultipartFile file) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<MultipleBrand> multipleBrandList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					MultipleBrand multipleBrand = new MultipleBrand();
					BinSection binSection = binSectionService.getBinSectionBySectionName(str[2]);
					if (null != binSection) {
						MultipleBrand obj = this.checkForExistence(str[0], OrganizationEvaluationRuleServiceImpl.toFullSpell(str[3]), binSection.getId(), str[6], str[7]);
						if (null != obj) {
							multipleBrand.setId(obj.getId());
							multipleBrand.setProfit(Double.valueOf(str[4]));
							multipleBrand.setScore(Double.valueOf(str[5]));
							multipleBrand.setRemark(str[8]);
							multipleBrandList.add(multipleBrand);
						} else {
							List<OrganizationStructure> organizationStructureList = organizationStructureService.getList2("", null, "");
							OrganizationStructure organizationStructure0 = organizationStructureList.stream().filter(item -> item.getShortName().equals(str[0])).findFirst().orElse(null);
							if (null != organizationStructure0) {
								multipleBrand.setOrganizationId(organizationStructure0.getId());
								multipleBrand.setOrganizationName(organizationStructure0.getShortName());
							}
							if (null != str[1] && !"".equals(str[1])) {
								OrganizationStructure organizationStructure1 = organizationStructureList.stream().filter(item -> item.getShortName().equals(str[1])).findFirst().orElse(null);
								if (null != organizationStructure1) {
									multipleBrand.setParentId(organizationStructure1.getId());
								}
							}
							multipleBrand.setBinSection(binSection.getId());
							multipleBrand.setCheckCycle(OrganizationEvaluationRuleServiceImpl.toFullSpell(str[3]));
							multipleBrand.setProfit(Double.valueOf(str[4]));
							multipleBrand.setScore(Double.valueOf(str[5]));
							multipleBrand.setYear(str[6]);
							multipleBrand.setMonth(str[7]);
							if (null != str[8]){
								multipleBrand.setRemark(str[8]);
							}
							multipleBrandList.add(multipleBrand);
						}
					}
				}
				boolean b = super.saveOrUpdateBatch(multipleBrandList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error("导入失败！");
				}
			} catch (Exception e) {
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.error("上传文件为空！");
	}

	@Override
	public MultipleBrand checkForExistence(String organizationName, String checkCycle, String binSection, String year, String month) {
		QueryWrapper<MultipleBrand> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(organizationName)) {
			qw.lambda().eq(MultipleBrand::getOrganizationName, organizationName);
		}
		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(MultipleBrand::getCheckCycle, checkCycle);
		}
		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(MultipleBrand::getBinSection, binSection);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(MultipleBrand::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(MultipleBrand::getMonth, month);
		}
		return baseMapper.selectOne(qw);
	}

	/**
	 * 转换为有树形结构的列表
	 *
	 * @param brands 源数据所有数据
	 * @return
	 */
	public List<MultipleBrand> convert(List<MultipleBrand> result, List<MultipleBrand> brands) {
		if (CollectionUtil.isEmpty(brands)) {
			return new ArrayList<>();
		}

		brands = brands.stream().filter(f -> !"0".equals(f.getParentId())).collect(Collectors.toList());
//		Map<String ,List<MultipleBrand>> brandMap = brands.stream().collect(Collectors.groupingBy(MultipleBrand::getParentId));

//		Map<String, List<MultipleBrand>> groupedBrands = brands.stream()
//			.collect(Collectors.groupingBy(brand -> brand.getParentId() + "_" + brand.getCheckCycle()));
		for (MultipleBrand brandsVO : result) {
			// 循环一级之后子级
			List<MultipleBrand> newbrands = null;
			if ("NDKP".equals(brandsVO.getCheckCycle())) {

				newbrands = brands.stream().filter(f -> brandsVO.getOrganizationId().equals(f.getParentId()) && brandsVO.getCheckCycle().equals(f.getCheckCycle())).collect(Collectors.toList());
			} else {
				newbrands = brands.stream().filter(f -> brandsVO.getOrganizationId().equals(f.getParentId()) && brandsVO.getCheckCycle().equals(f.getCheckCycle()) && brandsVO.getMonth().equals(f.getMonth())).collect(Collectors.toList());
			}
			brandsVO.setChildren(newbrands);
		}
		return result;
	}


	/**
	 * 循环设置子级
	 *
	 * @param list         所有元素列表
	 * @param treeSelectVO 父级元素
	 * @return 父级元素
	 */
	public static MultipleBrand getChildren(List<MultipleBrand> list, MultipleBrand treeSelectVO) {
		for (MultipleBrand MultipleBrand : list) {
			if (treeSelectVO.getOrganizationId().equals(MultipleBrand.getParentId())) {
				List<MultipleBrand> children = CollectionUtil.isEmpty(treeSelectVO.getChildren()) ? new ArrayList<>() : treeSelectVO.getChildren();
				children.add(getChildren(list, MultipleBrand));
				treeSelectVO.setChildren(children);

			}
		}
		return treeSelectVO;
	}

	private static void writeRow(Row row, MultipleBrand multipleBrand) {
		row.createCell(0).setCellValue(multipleBrand.getOrganizationName());
		row.createCell(1).setCellValue(multipleBrand.getBinSectionName());
		String checkCycle = "";
		if ("NDKP".equals(multipleBrand.getCheckCycle())) {
			checkCycle = "年度考评";
		} else if ("JDKP".equals(multipleBrand.getCheckCycle())) {
			checkCycle = "季度考评";
		} else if ("YDKP".equals(multipleBrand.getCheckCycle())) {
			checkCycle = "月度考评";
		}
		row.createCell(2).setCellValue(checkCycle);
		row.createCell(3).setCellValue(multipleBrand.getProfit());
		row.createCell(4).setCellValue(multipleBrand.getScore());
		row.createCell(5).setCellValue(multipleBrand.getYear());
		row.createCell(6).setCellValue(multipleBrand.getMonth());
		row.createCell(7).setCellValue(multipleBrand.getRemark());
	}
}
