package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.dao.PerformanceBenchmarkInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.service.*;
import lombok.Data;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * @author hlf
 * @date 2024/3/28 14:07
 * 文件说明：
 */
@Service
public class PerformanceBenchmarkInfoServiceImpl extends ServiceImpl<PerformanceBenchmarkInfoMapper, PerformanceBenchmarkInfo> implements IPerformanceBenchmarkInfoService {

	@Resource
	private IBenchmarkingRecordService benchmarkingRecordService;

	@Resource
	private IEvaluateRuleService evaluateRuleService;

	@Resource
	private IEvaluateRuleInfoService evaluateRuleInfoService;

	@Resource
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Resource
	private IOrganizationEvaluationInfoService organizationEvaluationInfoService;

	@Override
	public IPage<PerformanceBenchmarkInfo> listPage(Integer pageNum, Integer pageSize, String toolCategory, String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth, String indexName) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		int isBg = 0;
		int isDb = 0;
		int isCb = 0;
		if ("寻标".equals(toolCategory)) {
			xb(manageCategory, checkCycle, degreeYear, quarterlyMonth);
		} else if ("立标".equals(toolCategory)) {
			lb(manageCategory, checkCycle, degreeYear, quarterlyMonth);
		} else if ("标杆".equals(toolCategory)) {
			bg(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			isBg = 1;
		} else if ("达标".equals(toolCategory)) {
			db(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			isDb = 2;
		} else if ("超标".equals(toolCategory)) {
			cb(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			isCb = 3;
		}
		//构造分页构造器
		Page<PerformanceBenchmarkInfo> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, manageCategory, checkCycle, degreeYear, quarterlyMonth, indexName, isBg, isDb, isCb);
	}

	@Override
	public void exportExcel(String toolCategory, String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth, HttpServletResponse response) throws IOException {
		// 设置响应头信息，以附件形式下载
		response.setHeader("Content-Disposition", "attachment; filename=performance_benchmark_data.xlsx");
		response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		// 创建新的Excel工作簿
		Workbook workbook = new XSSFWorkbook();

		// 创建新的工作表
		Sheet sheet = workbook.createSheet("Sheet1");

		// 创建表头
		Row headerRow = sheet.createRow(0);
		List<String> headers = Arrays.asList("ID", "序号", "单位", "装机容量", "指标", "计划值", "实际完成值", "推荐值", "定标值", "是否标杆");
		for (int i = 0; i < headers.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellValue(headers.get(i));
		}

		//行数据
		int isBg = 0;
		int isDb = 0;
		int isCb = 0;
		if ("标杆".equals(toolCategory)) {
			isBg = 1;
		} else if ("达标".equals(toolCategory)) {
			isDb = 2;
		} else if ("超标".equals(toolCategory)) {
			isCb = 3;
		}
		List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.getExportExcelData(manageCategory, checkCycle, degreeYear, quarterlyMonth, isBg, isDb, isCb);

		// 填充数据到工作表
		int rowIndex = 1; // 从第二行开始填充数据（第一行是表头）
		for (PerformanceBenchmarkInfo performanceBenchmarkInfo : performanceBenchmarkInfoList) {
			Row row = sheet.createRow(rowIndex++);

			// 设置ID
			row.createCell(0).setCellValue(performanceBenchmarkInfo.getId());

			// 设置序号
			row.createCell(1).setCellValue(rowIndex - 1);

			// 设置其他数据
			row.createCell(2).setCellValue(performanceBenchmarkInfo.getCompanyShort());
			if (null != performanceBenchmarkInfo.getInstallCapacity()) {
				row.createCell(3).setCellValue(performanceBenchmarkInfo.getInstallCapacity());
			}
			row.createCell(4).setCellValue(performanceBenchmarkInfo.getIndexName());
			if (null != performanceBenchmarkInfo.getPlannedValue()) {
				row.createCell(5).setCellValue(performanceBenchmarkInfo.getPlannedValue());
			}
			if (null != performanceBenchmarkInfo.getCompleteValue()) {
				row.createCell(6).setCellValue(performanceBenchmarkInfo.getCompleteValue());
			}
			if (null != performanceBenchmarkInfo.getRecommendedValue()) {
				row.createCell(7).setCellValue(performanceBenchmarkInfo.getRecommendedValue());
			}
			if (null != performanceBenchmarkInfo.getScalingValue()) {
				row.createCell(8).setCellValue(performanceBenchmarkInfo.getScalingValue());
			}
			if (1 == performanceBenchmarkInfo.getIsBg()) {
				row.createCell(9).setCellValue("是");
			} else if (0 == performanceBenchmarkInfo.getIsBg()) {
				row.createCell(9).setCellValue("否");
			}
		}

		// 隐藏第一列（ID列）
		sheet.setColumnHidden(0, true);

		// 将工作簿写入到输出流
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
			workbook.write(outputStream);
			response.getOutputStream().write(outputStream.toByteArray());
			response.getOutputStream().flush();
		}

		// 关闭工作簿
		workbook.close();
	}

	/**
	 * 寻标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void xb(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "寻标");
		if (benchmarkingRecordList.size() == 0) {
			String zq = checkCycle;
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = new ArrayList<>();
			if ("YDKP".equals(checkCycle)) {
				checkCycle = "JDKP";
			}
			EvaluateRule evaluateRule = evaluateRuleService.getEvaluateRuleListByYear(String.valueOf(degreeYear), checkCycle, manageCategory);
			if (null != evaluateRule) {
				List<EvaluateRuleInfo> evaluateRuleInfoList = evaluateRuleInfoService.getEvaluateRuleInfoByEvaluateRuleId(evaluateRule.getId());
				for (EvaluateRuleInfo evaluateRuleInfo : evaluateRuleInfoList) {
					List<IndicatorDictionary> indicatorDictionaryList = indicatorDictionaryService.getIndicatorDictionaryList(evaluateRuleInfo.getIndicatorId());
					if (indicatorDictionaryList.size() > 0) {
						//计划值
						IndicatorDictionary jhz = indicatorDictionaryList.stream().filter(p -> !p.getOptionCode().equals("WCZ")).findFirst().orElse(null);
						//实际完成值
						IndicatorDictionary wcz = indicatorDictionaryList.stream().filter(p -> p.getOptionCode().equals("WCZ")).findFirst().orElse(null);
						if (jhz != null && wcz != null) {
							String qm;
							if (null != quarterlyMonth) {
								qm = String.valueOf(quarterlyMonth);
							} else {
								qm = "";
							}
							List<OrganizationEvaluationInfo> jhzList = organizationEvaluationInfoService.getOrganizationEvaluationInfoListByIndicatorDictionaryId(jhz.getId(), String.valueOf(degreeYear), qm);
							List<OrganizationEvaluationInfo> wczList = organizationEvaluationInfoService.getOrganizationEvaluationInfoListByIndicatorDictionaryId(wcz.getId(), String.valueOf(degreeYear), qm);
							if (jhzList.size() == wczList.size()) {
								for (int i = 0; i < wczList.size(); i++) {
									OrganizationEvaluationInfo jhzObj = jhzList.get(i);
									OrganizationEvaluationInfo wczObj = wczList.get(i);
									PerformanceBenchmarkInfo performanceBenchmarkInfo = new PerformanceBenchmarkInfo();
									performanceBenchmarkInfo.setId(Convert.toStr(IdUtil.getSnowflake(1, 1).nextId()));
									performanceBenchmarkInfo.setCheckCycle(wczObj.getCheckCycle());
									performanceBenchmarkInfo.setDegreeYear(degreeYear);
									if (!"NDKP".equals(checkCycle)) {
										performanceBenchmarkInfo.setQuarterlyMonth(quarterlyMonth);
									}
									performanceBenchmarkInfo.setManageCategory(manageCategory);
									performanceBenchmarkInfo.setCompanyId(wczObj.getOrganizationId());
									performanceBenchmarkInfo.setCompanyName(wczObj.getCompanyName());
									performanceBenchmarkInfo.setCompanyShort(wczObj.getCompanyShort());
									performanceBenchmarkInfo.setInstallCapacity(wczObj.getInstallCapacity());
									performanceBenchmarkInfo.setIndexName(evaluateRuleInfo.getIndicatorName());
									performanceBenchmarkInfo.setPlannedValue(jhzObj.getQuantifiedValue());
									performanceBenchmarkInfo.setCompleteValue(wczObj.getQuantifiedValue());
									performanceBenchmarkInfo.setIsBg(0);
									performanceBenchmarkInfo.setBgVal((double) 0);
									performanceBenchmarkInfo.setIsDb(0);
									performanceBenchmarkInfo.setDbVal((double) 0);
									performanceBenchmarkInfo.setIsCb(0);
									performanceBenchmarkInfo.setCbVal((double) 0);
									performanceBenchmarkInfoList.add(performanceBenchmarkInfo);
								}
							}
						}
					}
				}
			}
			super.saveBatch(performanceBenchmarkInfoList);
			if (performanceBenchmarkInfoList.size() > 0) {
				addRecord(manageCategory, zq, degreeYear, quarterlyMonth, "寻标");
			}
		}
	}

	/**
	 * 立标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void lb(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<PerformanceBenchmarkInfo> updateList = new ArrayList<>();
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "立标");
		if (benchmarkingRecordList.size() == 0) {
			if ("NDKP".equals(checkCycle)) {
				List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, null);
				//Map分组
				Map<String, List<PerformanceBenchmarkInfo>> grouped = performanceBenchmarkInfoList.stream().collect(Collectors.groupingBy(PerformanceBenchmarkInfo::getIndexName));
				//Map转List
				List<PbiGroup> pbiGroupList = grouped.entrySet().stream().map(obj -> new PbiGroup(obj.getKey(), obj.getValue())).collect(Collectors.toList());
				for (PbiGroup pbiGroup : pbiGroupList) {
					//指标完成值求和
					double completeValueSum = pbiGroup.getPbis().stream().filter(obj -> obj.getCompleteValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).average().orElse(0.0);
					//装机容量求和
					double installCapacitySum = pbiGroup.getPbis().stream().filter(obj -> obj.getInstallCapacity() >= 0).mapToDouble(PerformanceBenchmarkInfo::getInstallCapacity).average().orElse(0.0);
					//去年全年指标数据
					List<PerformanceBenchmarkInfo> wnData = baseMapper.selectListByIndexName(manageCategory, checkCycle, degreeYear - 1, null, pbiGroup.getIndexName());
					//前年全年指标数据
					List<PerformanceBenchmarkInfo> qnData = baseMapper.selectListByIndexName(manageCategory, checkCycle, degreeYear - 2, null, pbiGroup.getIndexName());
					double plannedValueWnSum;
					double plannedValueQnSum;
					double completeValueWnSum;
					if (qnData.size() > 0 && wnData.size() > 0) {
						//去年计划值
						plannedValueWnSum = wnData.stream().filter(obj -> obj.getPlannedValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getPlannedValue).average().orElse(0.0);
						//前年计划值
						plannedValueQnSum = qnData.stream().filter(obj -> obj.getPlannedValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getPlannedValue).average().orElse(0.0);
						//去年实际完成值
						completeValueWnSum = wnData.stream().filter(obj -> obj.getCompleteValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).average().orElse(0.0);
					} else if (wnData.size() > 0) {
						//去年计划值
						plannedValueWnSum = wnData.stream().filter(obj -> obj.getPlannedValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getPlannedValue).average().orElse(0.0);
						plannedValueQnSum = 0;
						completeValueWnSum = 0;
					} else {
						plannedValueWnSum = completeValueSum / pbiGroup.getPbis().size();
						plannedValueQnSum = -1;
						completeValueWnSum = -1;
					}
					pbiGroup.getPbis().forEach(PerformanceBenchmarkInfo -> {
						double A = yearIndexRecommendedValueA(completeValueSum, installCapacitySum, PerformanceBenchmarkInfo.getInstallCapacity());
						double B = yearIndexRecommendedValueB(plannedValueWnSum, plannedValueQnSum, completeValueWnSum);
						PerformanceBenchmarkInfo.setRecommendedValue(Math.max(A, B));
						updateList.add(PerformanceBenchmarkInfo);
					});
				}
			} else if ("JDKP".equals(checkCycle) || "YDKP".equals(checkCycle)) {
				List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
				//Map分组
				Map<String, List<PerformanceBenchmarkInfo>> grouped = performanceBenchmarkInfoList.stream().collect(Collectors.groupingBy(PerformanceBenchmarkInfo::getIndexName));
				//Map转List
				List<PbiGroup> pbiGroupList = grouped.entrySet().stream().map(entry -> new PbiGroup(entry.getKey(), entry.getValue())).collect(Collectors.toList());
				for (PbiGroup pbiGroup : pbiGroupList) {
					//去年1季/月度指标数据
					List<PerformanceBenchmarkInfo> wnfData = baseMapper.selectListByIndexName(manageCategory, checkCycle, degreeYear - 1, quarterlyMonth, pbiGroup.getIndexName());
					double zb;
					if (wnfData.size() > 0) {
						//去年全年指标数据
						List<PerformanceBenchmarkInfo> wnhData = baseMapper.selectListByIndexName(manageCategory, checkCycle, degreeYear - 1, null, pbiGroup.getIndexName());
						//去年全年发电量总和
						double wnhCompleteValueSum = wnhData.stream().filter(obj -> obj.getCompleteValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).sum();
						//去年1季/月度发电量总和
						double wnfCompleteValueSum = wnfData.stream().filter(obj -> obj.getCompleteValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).sum();
						//每月占比
						zb = wnfCompleteValueSum / wnhCompleteValueSum;
					} else {
						//当年全年指标数据
						List<PerformanceBenchmarkInfo> dnhData = baseMapper.selectListByIndexName(manageCategory, checkCycle, degreeYear, null, pbiGroup.getIndexName());
						//当年全年计划值总和
						double dnhCompleteValueSum = dnhData.stream().filter(obj -> obj.getCompleteValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).sum();
						//当年1季/月度计划值总和
						double dnfCompleteValueSum = pbiGroup.getPbis().stream().filter(obj -> obj.getCompleteValue() >= 0).mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).sum();
						zb = dnfCompleteValueSum / dnhCompleteValueSum;
					}
					performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
						if (null != PerformanceBenchmarkInfo.getPlannedValue()) {
							PerformanceBenchmarkInfo.setRecommendedValue(quarterlyMonthIndexRecommendedValue(PerformanceBenchmarkInfo.getPlannedValue(), zb));
							updateList.add(PerformanceBenchmarkInfo);
						}
					});
				}
			}
			super.updateBatchById(updateList);
			if (updateList.size() > 0) {
				addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "立标");
			}
		}
	}

	/**
	 * 标杆
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void bg(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "标杆");
		if (benchmarkingRecordList.size() == 0) {
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			Map<String, List<PerformanceBenchmarkInfo>> grouped = performanceBenchmarkInfoList.stream().collect(Collectors.groupingBy(PerformanceBenchmarkInfo::getIndexName));
			grouped.forEach((indexName, performanceBenchmarkInfos) -> {
				performanceBenchmarkInfos.forEach(PerformanceBenchmarkInfo -> {
					if (null != PerformanceBenchmarkInfo.getInstallCapacity() && null != PerformanceBenchmarkInfo.getCompleteValue() && 0 != PerformanceBenchmarkInfo.getCompleteValue()) {
						PerformanceBenchmarkInfo.setBgVal(PerformanceBenchmarkInfo.getInstallCapacity() / PerformanceBenchmarkInfo.getCompleteValue());
					}
				});
				performanceBenchmarkInfos.sort(Comparator.comparingDouble(PerformanceBenchmarkInfo::getBgVal).reversed());
				if (!performanceBenchmarkInfos.isEmpty() && performanceBenchmarkInfos.get(0).getBgVal() != 0) {
					performanceBenchmarkInfos.get(0).setIsBg(1);
				}
				super.updateBatchById(performanceBenchmarkInfos);
			});
			if (performanceBenchmarkInfoList.size() > 0) {
				addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "标杆");
			}
		}
	}

	/**
	 * 达标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void db(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "达标");
		if (benchmarkingRecordList.size() == 0) {
			boolean flag = false;
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			for (PerformanceBenchmarkInfo performanceBenchmarkInfo : performanceBenchmarkInfoList) {
				if (null != performanceBenchmarkInfo.getCompleteValue() && null != performanceBenchmarkInfo.getScalingValue()) {
					if (performanceBenchmarkInfo.getCompleteValue() >= performanceBenchmarkInfo.getScalingValue()) {
						performanceBenchmarkInfo.setIsDb(1);
						performanceBenchmarkInfo.setDbVal(performanceBenchmarkInfo.getCompleteValue() - performanceBenchmarkInfo.getScalingValue());
						flag = true;
					}
				} else {
					flag = false;
				}
			}
			super.updateBatchById(performanceBenchmarkInfoList);
			if (performanceBenchmarkInfoList.size() > 0 && flag) {
				addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "达标");
			}
		}
	}

	/**
	 * 超标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void cb(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "超标");
		if (benchmarkingRecordList.size() == 0) {
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			Map<String, List<PerformanceBenchmarkInfo>> grouped = performanceBenchmarkInfoList.stream().collect(Collectors.groupingBy(PerformanceBenchmarkInfo::getIndexName));
			AtomicBoolean flag = new AtomicBoolean(false);
			grouped.forEach((indexName, performanceBenchmarkInfos) -> {
				performanceBenchmarkInfos.stream().filter(p -> p.getIsBg().equals(1)).findFirst().ifPresent(performanceBenchmarkInfo -> performanceBenchmarkInfos.forEach(PerformanceBenchmarkInfo -> {
					if (null != PerformanceBenchmarkInfo.getCompleteValue() && null != performanceBenchmarkInfo.getScalingValue()) {
						if (PerformanceBenchmarkInfo.getCompleteValue() > performanceBenchmarkInfo.getScalingValue()) {
							PerformanceBenchmarkInfo.setIsCb(1);
							PerformanceBenchmarkInfo.setCbVal(PerformanceBenchmarkInfo.getCompleteValue() - performanceBenchmarkInfo.getScalingValue());
							flag.set(true);
						}
					} else {
						flag.set(false);
					}
				}));
				super.updateBatchById(performanceBenchmarkInfos);
			});
			if (performanceBenchmarkInfoList.size() > 0 && flag.get()) {
				addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "超标");
			}
		}
	}

	/**
	 * 年度指标推荐值A
	 */
	private Double yearIndexRecommendedValueA(double a, double b, double c) {
		return (a / b) * c;
	}

	/**
	 * 年度指标推荐值B
	 */
	private Double yearIndexRecommendedValueB(double a, double b, double c) {
		double B;
		if (b == -1) {
			B = a;
		} else if (b == 0) {
			B = a + (a * 0.02);
		} else {
			B = (a / b) * c;
		}
		return B;
	}

	/**
	 * 季/月度指标推荐值
	 */
	private Double quarterlyMonthIndexRecommendedValue(double a, double b) {
		return a * b;
	}

	private void addRecord(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth, String toolCategory) {
		BenchmarkingRecord benchmarkingRecord = new BenchmarkingRecord();
		benchmarkingRecord.setId(Convert.toStr(IdUtil.getSnowflake(1, 1).nextId()));
		benchmarkingRecord.setManageCategory(manageCategory);
		benchmarkingRecord.setCheckCycle(checkCycle);
		benchmarkingRecord.setDegreeYear(degreeYear);
		benchmarkingRecord.setQuarterlyMonth(quarterlyMonth);
		benchmarkingRecord.setTagType(toolCategory);
		benchmarkingRecordService.save(benchmarkingRecord);
	}

	@Data
	private static class PbiGroup {
		private String indexName;
		private List<PerformanceBenchmarkInfo> pbis;

		public PbiGroup(String indexName, List<PerformanceBenchmarkInfo> pbis) {
			this.indexName = indexName;
			this.pbis = pbis;
		}
	}
}
